<?php

namespace Drupal\mattermost_logger\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Psr\Log\LogLevel;

/**
 * The settings form for the Mattermost logger.
 */
class SettingsForm extends ConfigFormBase {
  const CHANNEL_WEBHOOKS = 'channel_webhooks';

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['mattermost_logger.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'mattermost_logger_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildForm($form, $form_state);

    $config = $this->config('mattermost_logger.settings');

    if ($form_state->has(self::CHANNEL_WEBHOOKS)) {
      $channel_webhooks = $form_state->get(self::CHANNEL_WEBHOOKS);
    }
    else {
      $channel_webhooks = $config->get(self::CHANNEL_WEBHOOKS) ?? [];
      $form_state->set(self::CHANNEL_WEBHOOKS, $channel_webhooks);
    }

    $form['webhook_url'] = [
      '#type' => 'url',
      '#title' => $this->t('Default mattermost webhook URL'),
      '#description' => $this->t('The default Mattermost webhook URL to send messages to.'),
      '#default_value' => $config->get('webhook_url'),
    ];

    $form[self::CHANNEL_WEBHOOKS] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Channel webhooks'),
      '#prefix' => '<div id="channel-webhooks-wrapper">',
      '#suffix' => '</div>',
      '#description' => $this->t('Configure webhooks for specific logging channels or enter a custom channel name with its webhook URL.'),
    ];

    foreach ($channel_webhooks as $index => $channel_webhook) {
      $form[self::CHANNEL_WEBHOOKS][$index] = [
        'channel' => [
          '#type' => 'textfield',
          '#title' => $this->t('Logging channel'),
          '#default_value' => $channel_webhook['channel'] ?? '',
          '#parents' => [self::CHANNEL_WEBHOOKS, $index, 'channel'],
          '#required' => TRUE,
        ],
        'webhook_url' => [
          '#type' => 'url',
          '#title' => $this->t('Mattermost webhook URL'),
          '#default_value' => $channel_webhook['webhook_url'] ?? '',
          '#parents' => [self::CHANNEL_WEBHOOKS, $index, 'webhook_url'],
          '#states' => [
            'visible' => [
              ':input[name="' . self::CHANNEL_WEBHOOKS . "[$index][use_default_webhook_url]\"]" => ['checked' => FALSE],
            ],
            'required' => [
              ':input[name="' . self::CHANNEL_WEBHOOKS . "[$index][use_default_webhook_url]\"]" => ['checked' => FALSE],
            ],
            'enabled' => [
              ':input[name="' . self::CHANNEL_WEBHOOKS . "[$index][use_default_webhook_url]\"]" => ['checked' => FALSE],
            ],
          ],
        ],
        'use_default_webhook_url' => [
          '#type' => 'checkbox',
          '#title' => $this->t('Use default webhook url'),
          '#default_value' => $channel_webhook['use_default_webhook_url'] ?? '',
          '#parents' => [self::CHANNEL_WEBHOOKS, $index, 'use_default_webhook_url'],
        ],
        'level' => [
          '#type' => 'checkboxes',
          '#title' => $this->t('Log level'),
          '#options' => [
            LogLevel::EMERGENCY => $this->t('Emergency'),
            LogLevel::ALERT => $this->t('Alert'),
            LogLevel::CRITICAL => $this->t('Critical'),
            LogLevel::ERROR => $this->t('Error'),
            LogLevel::WARNING => $this->t('Warning'),
            LogLevel::NOTICE => $this->t('Notice'),
            LogLevel::INFO => $this->t('Info'),
            LogLevel::DEBUG => $this->t('Debug'),
          ],
          '#default_value' => $channel_webhook['level'] ?? [],
          '#parents' => [self::CHANNEL_WEBHOOKS, $index, 'level'],
        ],

        'remove' => [
          '#type' => 'submit',
          '#value' => $this->t('Remove'),
          '#submit' => ['::removeWebhook'],
          '#name' => 'remove_' . $index,
           // Skip validation.
          '#limit_validation_errors' => [],
          '#ajax' => [
            'callback' => '::ajaxRefresh',
            'wrapper' => 'channel-webhooks-wrapper',
          ],
        ],

        'separator' => [
          '#markup' => '<hr>',
        ],
      ];
    }

    $form[self::CHANNEL_WEBHOOKS]['add_webhook'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add channel webhook'),
      '#submit' => ['::addWebhook'],
      '#ajax' => [
        'callback' => '::ajaxRefresh',
        'wrapper' => 'channel-webhooks-wrapper',
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $channel_webhooks = $form_state->getValue(self::CHANNEL_WEBHOOKS) ?? [];

    foreach ($channel_webhooks as &$webhook) {
      if (!empty($webhook['use_default_webhook_url'])) {
        unset($webhook['webhook_url']);
      }
      if (!empty($webhook['level'])) {
        $webhook['level'] = array_filter($webhook['level']);
      }
    }

    $this->config('mattermost_logger.settings')
      ->set('webhook_url', $form_state->getValue('webhook_url'))
      ->set(self::CHANNEL_WEBHOOKS, array_values($channel_webhooks))
      ->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * Add a webhook to the form.
   */
  public function addWebhook(array &$form, FormStateInterface $form_state) {
    $channel_webhooks = $form_state->get(self::CHANNEL_WEBHOOKS) ?? [];

    $channel_webhooks[] = [
      'channel' => '',
      'webhook_url' => '',
      'use_default_webhook_url' => FALSE,
      'level' => [],
    ];

    $form_state->set(self::CHANNEL_WEBHOOKS, $channel_webhooks);
    $form_state->setRebuild();
  }

  /**
   * Remove a webhook from the form.
   */
  public function removeWebhook(array &$form, FormStateInterface $form_state) {
    $triggering_element = $form_state->getTriggeringElement();
    $index = str_replace('remove_', '', $triggering_element['#name']);

    $channel_webhooks = $form_state->get(self::CHANNEL_WEBHOOKS);
    unset($channel_webhooks[$index]);
    $form_state->set(self::CHANNEL_WEBHOOKS, $channel_webhooks);
    $form_state->setRebuild();
  }

  /**
   * Ajax callback to refresh the channel webhooks fieldset.
   */
  public function ajaxRefresh(array &$form, FormStateInterface $form_state) {
    return $form[self::CHANNEL_WEBHOOKS];
  }

}
