<?php

namespace Drupal\mattermost_logger;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\Core\Logger\LogMessageParserInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use GuzzleHttp\Client;
use Psr\Log\LogLevel;

/**
 * Mattermost logger.
 */
class MattermostLogger {
  use LoggerChannelTrait;
  use StringTranslationTrait;

  /**
   * The webhook URL.
   *
   * @var ?string
   */
  protected ?string $webhookUrl;

  public function __construct(protected Client $httpClient, protected ConfigFactoryInterface $configFactory, protected LogMessageParserInterface $parser) {
    $this->webhookUrl = $this->configFactory->get('mattermost_logger.settings')->get('webhook_url');
  }

  /**
   * Send a message to Mattermost.
   *
   * @param string $channel
   *   The name of the module that is sending the message.
   * @param string $message
   *   The message to send.
   * @param string $level
   *   The type of level to send. (default: INFO)
   * @param array $context
   *   Additional context to send, only used for placeholders.
   * @param array $optional
   *   Optional extra payload options to customize the message, such as advanced
   *   attachments. Supports the following structure:
   *   - attachments: (array[]) List of attachment objects. Supports:
   *     - fallback: (string) Required plain-text summary of the attachment.
   *     - color: (string) Hex color code for the left border (e.g. "#FF8000").
   *     - pretext: (string) Optional text that appears above the attachment.
   *     - text: (string) Main text of the attachment.
   *     - author_name: (string) Display name of the author.
   *     - author_icon: (string) URL to the author's icon image.
   *     - author_link: (string) URL to link the author name.
   *     - title: (string) Title of the attachment, usually bolded.
   *     - title_link: (string) URL the title links to.
   *     - fields: (array[]) Array of field objects, each with:
   *       - short: (bool) Whether the field should be displayed in a
   *       short (column) format.
   *       - title: (string) Title of the field.
   *       - value: (string) Value of the field.
   *     - image_url: (string) URL of an image to display below the text.
   */
  public function sendMessage(string $channel, string $message, string $level, array $context = [], array $optional = []): void {
    if (!$this->webhookUrl) {
      $this->getLogger('mattermost_logger')
        ->error($this->t('No webhook URL configured. Unable to log Mattermost message for @channel.',
          ['@channel' => $channel]));
      return;
    }

    $message_placeholders = $this->parser->parseMessagePlaceholders($message, $context);
    $message = empty($message_placeholders) ? $message : strtr($message, $message_placeholders);

    $mattermost_emoji = match ($level) {
      LogLevel::ERROR, LogLevel::EMERGENCY => ':x:',
      LogLevel::ALERT => ':bellhop_bell:',
      LogLevel::WARNING => ':warning:',
      default => ':information_source:',
    };
    $color = match ($level) {
      LogLevel::ERROR, LogLevel::EMERGENCY => '#FF0000',
      LogLevel::ALERT => '#FFA500',
      LogLevel::WARNING => '#FFFF00',
      default => '#3E76FF',
    };

    $level = strtoupper($level);
    $text = "$mattermost_emoji [$channel] **$level:** \n$message";

    $payload = [
      'attachments' => [
        [
          'text' => $text,
          'color' => $color,
          'mrkdwn_in' => ['text'],
          ...$optional,
        ],
      ],
    ];

    try {
      $this->httpClient->post($this->webhookUrl, [
        'json' => $payload,
      ]);
    }
    catch (\Exception $e) {
      $this->getLogger('mattermost_logger')->error($this->t('Failed to send Mattermost notification for @channel: @message', [
        '@channel' => $channel,
        '@message' => $e->getMessage(),
      ]));
    }
  }

  /**
   * Set the webhook URL.
   */
  public function setWebhookUrl(string $webhook_url): void {
    $this->webhookUrl = $webhook_url;
  }

  /**
   * Sends an error message to Mattermost.
   *
   * @param string $channel
   *   The name of the module that is sending the message.
   * @param string $message
   *   The message to send.
   */
  public function error(string $channel, string $message): void {
    $this->sendMessage($channel, $message, LogLevel::ERROR);
  }

  /**
   * Sends a warning message to Mattermost.
   *
   * @param string $channel
   *   The name of the module that is sending the message.
   * @param string $message
   *   The message to send.
   */
  public function warning(string $channel, string $message): void {
    $this->sendMessage($channel, $message, LogLevel::WARNING);
  }

  /**
   * Sends an alert message to Mattermost.
   *
   * @param string $channel
   *   The name of the module that is sending the message.
   * @param string $message
   *   The message to send.
   */
  public function alert(string $channel, string $message): void {
    $this->sendMessage($channel, $message, LogLevel::ALERT);
  }

  /**
   * Sends a notice message to Mattermost.
   *
   * @param string $channel
   *   The name of the module that is sending the message.
   * @param string $message
   *   The message to send.
   */
  public function notice(string $channel, string $message): void {
    $this->sendMessage($channel, $message, LogLevel::NOTICE);
  }

  /**
   * Sends an info message to Mattermost.
   *
   * @param string $channel
   *   The name of the module that is sending the message.
   * @param string $message
   *   The message to send.
   */
  public function info(string $channel, string $message): void {
    $this->sendMessage($channel, $message, LogLevel::INFO);
  }

  /**
   * Sends a debug message to Mattermost.
   *
   * @param string $channel
   *   The name of the module that is sending the message.
   * @param string $message
   *   The message to send.
   */
  public function debug(string $channel, string $message): void {
    $this->sendMessage($channel, $message, LogLevel::DEBUG);
  }

  /**
   * Sends an emergency message to Mattermost.
   *
   * @param string $channel
   *   The name of the module that is sending the message.
   * @param string $message
   *   The message to send.
   */
  public function emergency(string $channel, string $message): void {
    $this->sendMessage($channel, $message, LogLevel::EMERGENCY);
  }

  /**
   * Sends a critical message to Mattermost.
   *
   * @param string $channel
   *   The name of the module that is sending the message.
   * @param string $message
   *   The message to send.
   */
  public function critical(string $channel, string $message): void {
    $this->sendMessage($channel, $message, LogLevel::CRITICAL);
  }

}
